@@ -1,5 +1,6 @@
 from pkg_resources import resource_filename
 from functools import lru_cache
+from typing import List, Dict, Iterator, Tuple
 import langcodes
 import msgpack
 import gzip
@@ -12,12 +13,13 @@ import warnings
 
 from .tokens import tokenize, simple_tokenize, lossy_tokenize
 from .language_info import get_language_info
+from .numbers import digit_freq, has_digit_sequence, smash_numbers
 
 logger = logging.getLogger(__name__)
 
 
 CACHE_SIZE = 100000
-DATA_PATH = pathlib.Path(resource_filename('wordfreq', 'data'))
+DATA_PATH = pathlib.Path(resource_filename("wordfreq", "data"))
 
 # We'll divide the frequency by 10 for each token boundary that was inferred.
 # (We determined the factor of 10 empirically by looking at words in the
@@ -33,7 +35,7 @@ tokenize = tokenize
 simple_tokenize = simple_tokenize
 
 
-def read_cBpack(filename):
+def read_cBpack(filename: str) -> List[List[str]]:
     """
     Read a file from an idiosyncratic format that we use for storing
     approximate word frequencies, called "cBpack".
@@ -74,44 +76,45 @@ def read_cBpack(filename):
             ['blue', 'red']
         ]
     """
-    with gzip.open(filename, 'rb') as infile:
+    with gzip.open(filename, "rb") as infile:
         data = msgpack.load(infile, raw=False)
     header = data[0]
     if (
-        not isinstance(header, dict) or header.get('format') != 'cB'
-        or header.get('version') != 1
+        not isinstance(header, dict)
+        or header.get("format") != "cB"
+        or header.get("version") != 1
     ):
         raise ValueError("Unexpected header: %r" % header)
     return data[1:]
 
 
-def available_languages(wordlist='best'):
+def available_languages(wordlist: str = "best") -> Dict[str, str]:
     """
     Given a wordlist name, return a dictionary of language codes to filenames,
     representing all the languages in which that wordlist is available.
     """
-    if wordlist == 'best':
-        available = available_languages('small')
-        available.update(available_languages('large'))
+    if wordlist == "best":
+        available = available_languages("small")
+        available.update(available_languages("large"))
         return available
-    elif wordlist == 'combined':
-        logger.warning(
-            "The 'combined' wordlists have been renamed to 'small'."
-        )
-        wordlist = 'small'
+    elif wordlist == "combined":
+        logger.warning("The 'combined' wordlists have been renamed to 'small'.")
+        wordlist = "small"
 
     available = {}
-    for path in DATA_PATH.glob('*.msgpack.gz'):
-        if not path.name.startswith('_'):
-            list_name = path.name.split('.')[0]
-            name, lang = list_name.split('_')
+    for path in DATA_PATH.glob("*.msgpack.gz"):
+        if not path.name.startswith("_"):
+            list_name = path.name.split(".")[0]
+            name, lang = list_name.split("_")
             if name == wordlist:
                 available[lang] = str(path)
     return available
 
 
 @lru_cache(maxsize=None)
-def get_frequency_list(lang, wordlist='best', match_cutoff=None):
+def get_frequency_list(
+    lang: str, wordlist: str = "best", match_cutoff: None = None
+) -> List[List[str]]:
     """
     Read the raw data from a wordlist file, returning it as a list of
     lists. (See `read_cBpack` for what this represents.)
@@ -122,33 +125,25 @@ def get_frequency_list(lang, wordlist='best', match_cutoff=None):
     Looking up the alternate code 'por' will also get the same list.
     """
     if match_cutoff is not None:
-        warnings.warn(
-            "The `match_cutoff` parameter is deprecated",
-            DeprecationWarning
-        )
+        warnings.warn("The `match_cutoff` parameter is deprecated", DeprecationWarning)
     available = available_languages(wordlist)
 
-    # TODO: decrease the maximum distance. This distance is so high just
-    # because it allows a test where 'yue' matches 'zh', and maybe the
-    # distance between those is high because they shouldn't match.
-    best, _distance = langcodes.closest_match(
-        lang, list(available), max_distance=70
-    )
-    if best == 'und':
-        raise LookupError("No wordlist %r available for language %r"
-                          % (wordlist, lang))
+    # the max_distance is high because we unify scripts, such as Traditional
+    # vs. Simplified Chinese, in one wordlist
+    best, _distance = langcodes.closest_match(lang, list(available), max_distance=60)
+    if best == "und":
+        raise LookupError("No wordlist %r available for language %r" % (wordlist, lang))
 
     if best != lang:
         logger.warning(
             "You asked for word frequencies in language %r. Using the "
-            "nearest match, which is %r."
-            % (lang, best)
+            "nearest match, which is %r." % (lang, best)
         )
 
     return read_cBpack(available[best])
 
 
-def cB_to_freq(cB):
+def cB_to_freq(cB: int) -> float:
     """
     Convert a word frequency from the logarithmic centibel scale that we use
     internally, to a proportion from 0 to 1.
@@ -160,13 +155,11 @@ def cB_to_freq(cB):
     In general, x cB represents a frequency of 10 ** (x/100).
     """
     if cB > 0:
-        raise ValueError(
-            "A frequency cannot be a positive number of centibels."
-        )
+        raise ValueError("A frequency cannot be a positive number of centibels.")
     return 10 ** (cB / 100)
 
 
-def cB_to_zipf(cB):
+def cB_to_zipf(cB: int) -> float:
     """
     Convert a word frequency from centibels to the Zipf scale
     (see `zipf_to_freq`).
@@ -178,7 +171,7 @@ def cB_to_zipf(cB):
     return (cB + 900) / 100
 
 
-def zipf_to_freq(zipf):
+def zipf_to_freq(zipf: float) -> float:
     """
     Convert a word frequency from the Zipf scale to a proportion between 0 and
     1.
@@ -191,10 +184,10 @@ def zipf_to_freq(zipf):
     words. For example, a word that occurs once per million words is at 3.0 on
     the Zipf scale.
     """
-    return 10 ** zipf / 1e9
+    return 10**zipf / 1e9
 
 
-def freq_to_zipf(freq):
+def freq_to_zipf(freq: float) -> float:
     """
     Convert a word frequency from a proportion between 0 and 1 to the
     Zipf scale (see `zipf_to_freq`).
@@ -203,16 +196,15 @@ def freq_to_zipf(freq):
 
 
 @lru_cache(maxsize=None)
-def get_frequency_dict(lang, wordlist='best', match_cutoff=None):
+def get_frequency_dict(
+    lang: str, wordlist: str = "best", match_cutoff: None = None
+) -> Dict[str, float]:
     """
     Get a word frequency list as a dictionary, mapping tokens to
     frequencies as floating-point probabilities.
     """
     if match_cutoff is not None:
-        warnings.warn(
-            "The `match_cutoff` parameter is deprecated",
-            DeprecationWarning
-        )
+        warnings.warn("The `match_cutoff` parameter is deprecated", DeprecationWarning)
     freqs = {}
     pack = get_frequency_list(lang, wordlist)
     for index, bucket in enumerate(pack):
@@ -222,7 +214,7 @@ def get_frequency_dict(lang, wordlist='best', match_cutoff=None):
     return freqs
 
 
-def iter_wordlist(lang, wordlist='best'):
+def iter_wordlist(lang: str, wordlist: str = "best") -> Iterator[str]:
     """
     Yield the words in a wordlist in approximate descending order of
     frequency.
@@ -237,11 +229,12 @@ def iter_wordlist(lang, wordlist='best'):
 # This dict and inner function are used to implement a "drop everything" cache
 # for word_frequency(); the overheads of lru_cache() are comparable to the time
 # it takes to look up frequencies from scratch, so something faster is needed.
-_wf_cache = {}
+_wf_cache: Dict[Tuple[str, str, str, float], float] = {}
 
 
-def _word_frequency(word, lang, wordlist, minimum):
+def _word_frequency(word: str, lang: str, wordlist: str, minimum: float) -> float:
     tokens = lossy_tokenize(word, lang)
+
     if not tokens:
         return minimum
 
@@ -252,14 +245,23 @@ def _word_frequency(word, lang, wordlist, minimum):
     freqs = get_frequency_dict(lang, wordlist)
     one_over_result = 0.0
     for token in tokens:
-        if token not in freqs:
+        smashed = smash_numbers(token)
+        if smashed not in freqs:
             # If any word is missing, just return the default value
             return minimum
-        one_over_result += 1.0 / freqs[token]
+        freq = freqs[smashed]
+        if smashed != token:
+            # If there is a digit sequence in the token, the digits are
+            # internally replaced by 0s to aggregate their probabilities
+            # together. We then assign a specific frequency to the digit
+            # sequence using the `digit_freq` distribution.
+            freq *= digit_freq(token)
+        one_over_result += 1.0 / freq
 
+    # Combine the frequencies of tokens we looked up.
     freq = 1.0 / one_over_result
 
-    if get_language_info(lang)['tokenizer'] == 'jieba':
+    if get_language_info(lang)["tokenizer"] == "jieba":
         # If we used the Jieba tokenizer, we could tokenize anything to match
         # our wordlist, even nonsense. To counteract this, we multiply by a
         # probability for each word break that was inferred.
@@ -268,14 +270,16 @@ def _word_frequency(word, lang, wordlist, minimum):
     # All our frequency data is only precise to within 1% anyway, so round
     # it to 3 significant digits
     unrounded = max(freq, minimum)
-    if unrounded == 0.:
-        return 0.
+    if unrounded == 0.0:
+        return 0.0
     else:
         leading_zeroes = math.floor(-math.log(unrounded, 10))
         return round(unrounded, leading_zeroes + 3)
 
 
-def word_frequency(word, lang, wordlist='best', minimum=0.):
+def word_frequency(
+    word: str, lang: str, wordlist: str = "best", minimum: float = 0.0
+) -> float:
     """
     Get the frequency of `word` in the language with code `lang`, from the
     specified `wordlist`.
@@ -302,7 +306,7 @@ def word_frequency(word, lang, wordlist='best', minimum=0.):
         return _wf_cache[args]
 
 
-def zipf_frequency(word, lang, wordlist='best', minimum=0.):
+def zipf_frequency(word: str, lang: str, wordlist: str = "best", minimum: float = 0.0):
     """
     Get the frequency of `word`, in the language with code `lang`, on the Zipf
     scale.
@@ -330,23 +334,35 @@ def zipf_frequency(word, lang, wordlist='best', minimum=0.):
 
 
 @lru_cache(maxsize=100)
-def top_n_list(lang, n, wordlist='best', ascii_only=False):
+def top_n_list(
+    lang: str, n: int, wordlist: str = "best", ascii_only: bool = False
+) -> List[str]:
     """
     Return a frequency list of length `n` in descending order of frequency.
     This list contains words from `wordlist`, of the given language.
     If `ascii_only`, then only ascii words are considered.
+
+    The frequency list will not contain multi-digit sequences, because we
+    estimate the frequencies of those using the functions in `numbers.py`,
+    not using a wordlist that contains all of them.
     """
     results = []
     for word in iter_wordlist(lang, wordlist):
-        if (not ascii_only) or max(word) <= '~':
-            results.append(word)
-            if len(results) >= n:
-                break
+        if (not ascii_only) or max(word) <= "~":
+            if not has_digit_sequence(word):
+                results.append(word)
+                if len(results) >= n:
+                    break
     return results
 
 
-def random_words(lang='en', wordlist='best', nwords=5, bits_per_word=12,
-                 ascii_only=False):
+def random_words(
+    lang: str = "en",
+    wordlist: str = "best",
+    nwords: int = 5,
+    bits_per_word: int = 12,
+    ascii_only: bool = False,
+) -> str:
     """
     Returns a string of random, space separated words.
 
@@ -360,18 +376,19 @@ def random_words(lang='en', wordlist='best', nwords=5, bits_per_word=12,
     You can restrict the selection of words to those written in ASCII
     characters by setting `ascii_only` to True.
     """
-    n_choices = 2 ** bits_per_word
+    n_choices = 2**bits_per_word
     choices = top_n_list(lang, n_choices, wordlist, ascii_only=ascii_only)
     if len(choices) < n_choices:
         raise ValueError(
             "There aren't enough words in the wordlist to provide %d bits of "
             "entropy per word." % bits_per_word
         )
-    return ' '.join([random.choice(choices) for i in range(nwords)])
+    return " ".join([random.choice(choices) for i in range(nwords)])
 
 
-def random_ascii_words(lang='en', wordlist='best', nwords=5,
-                       bits_per_word=12):
+def random_ascii_words(
+    lang: str = "en", wordlist: str = "best", nwords: int = 5, bits_per_word: int = 12
+) -> str:
     """
     Returns a string of random, space separated, ASCII words.
 
